<?php

declare(strict_types=1);

namespace Drupal\radioactivity\Hook;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Hook\Attribute\Hook;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\StringTranslation\TranslationInterface;
use Drupal\Core\Entity\EntityFieldManagerInterface;

/**
 * Hook implementations used to alter and enhance forms.
 */
final class RadioactivityFormHooks {
  use StringTranslationTrait;

  /**
   * Constructs a new RadioactivityFormHooks service.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   The config.factory service.
   * @param \Drupal\Core\StringTranslation\TranslationInterface $string_translation
   *   The string translation service.
   * @param \Drupal\Core\Entity\EntityFieldManagerInterface $entityFieldManager
   *   The entity field manager service.
   */
  public function __construct(
    protected ConfigFactoryInterface $configFactory,
    TranslationInterface $string_translation,
    protected EntityFieldManagerInterface $entityFieldManager,
  ) {
    $this->stringTranslation = $string_translation;
  }

  /**
   * Implements hook_form_FORM_ID_alter() for 'field_config_edit_form'.
   */
  #[Hook('form_field_config_edit_form_alter')]
  public function fieldConfigEditFormAlter(array &$form, FormStateInterface $form_state): void {
    $fieldConfig = $form_state->getFormObject()->getEntity();
    if ($fieldConfig->getType() !== 'radioactivity_reference') {
      return;
    }

    if (isset($form['required'])) {
      $form['required']['#disabled'] = TRUE;
      $form['required']['#default_value'] = TRUE;
    }
  }

  /**
   * Implements hook_form_FORM_ID_alter() for 'field_ui_field_storage_add_form'.
   *
   * @todo Remove in 5.0.0 when the deprecated 'radioactivity' field gets removed.
   */
  #[Hook('form_field_ui_field_storage_add_form_alter')]
  public function fieldUiFieldStorageAddFormAlter(array &$form, FormStateInterface $form_state, string $form_id): void {
    // Hide the deprecated radioactivity field type when no field of this type
    // is currently being used.
    $map = $this->entityFieldManager->getFieldMapByFieldType('radioactivity');
    if (empty($map)) {
      unset($form['add']['new_storage_type']['#options']['General']['radioactivity']);
    }
  }

}
